#include "CDEV.h"

// An sample cdev created using the CW CDEV Framework
// By: Matthew E. Axsom
//--------------------------
// Version History
// 1.0 	-	12/09/94 Initial release

// While it's not the most complicated cdev in the world, it
// hopefully will give you the ideas behind the framework.


// control panel items
enum {
	kUserItem=1,
	kCheckBox,
	kTextBox=4
};

// base resource id
enum {
	kBaseID=-4064
};

// items in STR# resource
enum {
	kDeactivatedText=1,
	kAlertText,
	kInitialText
};

// prototypes for 2 required procedures
long runable(void);
cdevObj *makeCDEV(short numItems,DialogPtr cp);

// return 1 so we can run.
long runable(void)
{
	return 1;
}

// code that allocates our cdev
cdevObj *makeCDEV(short numItems,DialogPtr cp)
{
	// all we need to do here is allocate *our* object that controls the cdev
	return new myCDevObj(numItems,cp);
}

// constructor for our cdev
myCDevObj::myCDevObj(short numItems,DialogPtr cp) : cdevObj(numItems,cp)
{
	// set up our internal members
	activated=false;
	lastTime=0;
	showSeconds=true;
	
	// set the check box up
	setCheckBox();
	
	// put the initial text in the edit text box
	
	// get initial string
	Str255	s;
	GetIndString(s,kBaseID,kInitialText);

	// get the handle to the text, install and select all the text
	Handle	h;
	Rect	r;
	short	type;
	
	GetDItem(d,lastItem+kTextBox,&type,&h,&r);
	SetIText(h,s);
	SelIText(d,lastItem+kTextBox,0,32000);
}

// destructor for our cdev
myCDevObj::~myCDevObj(void)
{
	// I have no dynamic storage that I need to get rid of before we close
}

// handles a mouse hit in the control panel
long myCDevObj::hit(short itemHit)
{
	// the only hitable item I need to worry about is the checkbox...
	switch (itemHit) {
		case kCheckBox:
			// modify our internal member to the checkbox's new state and update the check box
			showSeconds=!showSeconds;
			setCheckBox();
			
			// forces a redraw on the next idle
			lastTime=0;	
			break;
	}
	return noErr;
}

// handling of nulDev message
long myCDevObj::idle(void)
{
	// only do this if we are in the forground
	if (activated)
		// if the time has changed, do an update
		if (getTime())
			update();
		
	return noErr;
}

// update user items
long myCDevObj::update(void)
{
	Handle	h;
	Rect	r;
	short	type;
	GrafPtr	savePort;
	
	// since this can get called from a variety of places, I want to make sure that
	// we are drawing into the correct window.
	GetPort(&savePort);
	SetPort(d);
	
	// get and draw the time sting
	GetDItem(d,lastItem+kUserItem,&type,&h,&r);
	TextBox(&timeString[1],timeString[0],&r,teFlushDefault);
	
	// restore the original drawing port
	SetPort(savePort);
	
	return noErr;
}

// activate user items
long myCDevObj::activate(void)
{
	activated=true;	// we are being brought into the forground.
	lastTime=0;		// force us to update on the next idle call
	return noErr;
}

// deactivate user items
long myCDevObj::deactivate(void)
{
	activated=false;	// we are being pushed into the background
	
	// get the deactivated text and draw it
	GetIndString(timeString,kBaseID,kDeactivatedText);
	update();
	
	return noErr;
}

// undo from edit menu or cmd-z
long myCDevObj::undo(void)		
{
	Str255	s;
	
	// pop up an alert letting the user know that we don't support undo
	GetIndString(s,kBaseID,kAlertText);
	ParamText(s,nil,nil,nil);
	NoteAlert(kBaseID+1,nil);
	
	return noErr;
}

// gets the current time and puts it into a string if it's not the same as the time
// we currently have
Boolean myCDevObj::getTime(void)
{
	unsigned long	nuTime;
	Boolean			result=false;	// assume no update
	
	// get the time it is now
	GetDateTime(&nuTime);
	
	// if it's different than the last posted time then...
	if (nuTime != lastTime) {
	
		// create a time string based on the users settings
		IUTimeString(nuTime,showSeconds,timeString);
		
		// lastTime is now the current time
		lastTime=nuTime;
		
		// let the caller know that we updated the time
		result=true;
	}
	
	return result;
}

// check and unchecks the checkbox
void myCDevObj::setCheckBox(void)
{
	short	type;
	Handle	h;
	Rect	r;
	
	// get the control handle to the checkbox and set to to whatever
	// showSeconds is.
	GetDItem(d,lastItem+kCheckBox,&type,&h,&r);
	SetCtlValue((ControlHandle)h,showSeconds);
}
